==> dynamodb-data-operation-1.0.45.tgz: package/dist-src/core/dynamo-filter-query-operation.js <==
import { LoggingService } from "../helpers/logging-service";
import { UtilService } from "../helpers/util-service";
const conditionKeyMap = {
    $eq: "=",
    $notEq: "<>",
    $lt: "<",
    $lte: "<=",
    $gt: ">",
    $gte: ">=",
    $exists: "",
    $notExists: "",
    $in: "",
    $between: "",
    $contains: "",
    $notContains: "",
    $beginsWith: "",
};
function hasQueryConditionKey(key) {
    const queryCondKeys = Object.keys(conditionKeyMap);
    return queryCondKeys.includes(key);
}
const getRandom = () => [Math.round(Math.random() * 99999), Math.round(Math.random() * 88888), Math.round(Math.random() * 99)].join("");
export class DynamoFilterQueryOperation {
    operation__filterFieldExist({ fieldName }) {
        const attrKeyHash = `#attrKey1${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeNames: {
                [attrKeyHash]: fieldName,
            },
            xFilterExpression: `attribute_exists (${attrKeyHash})`,
        };
        return result;
    }
    operation__filterFieldNotExist({ fieldName }) {
        const attrKeyHash = `#attrKey2${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeNames: {
                [attrKeyHash]: fieldName,
            },
            xFilterExpression: `attribute_not_exists (${attrKeyHash})`,
        };
        return result;
    }
    ddo__helperFilterBasic({ fieldName, val, conditionExpr, }) {
        const keyAttr = `:attr${fieldName}${getRandom()}`.toLowerCase();
        const attrKeyHash = `#attrKey3${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeValues: {
                [keyAttr]: val,
            },
            xExpressionAttributeNames: {
                [attrKeyHash]: fieldName,
            },
            xFilterExpression: [attrKeyHash, conditionExpr, keyAttr].join(" "),
        };
        return result;
    }
    operation__filterIn({ fieldName, attrValues }) {
        const expressAttrVal = {};
        const expressAttrName = {};
        const filterExpress = [];
        const _attrKeyHash = `#attrKey4${getRandom()}`.toLowerCase();
        expressAttrName[_attrKeyHash] = fieldName;
        attrValues.forEach((item) => {
            const keyAttr = `:attr${fieldName}${getRandom()}`.toLowerCase();
            expressAttrVal[keyAttr] = item;
            filterExpress.push(`${_attrKeyHash} = ${keyAttr}`);
        });
        const _filterExpression = filterExpress.join(" OR ").trim();
        const _filterExpressionValue = filterExpress.length > 1 ? `(${_filterExpression})` : _filterExpression;
        const result = {
            xExpressionAttributeValues: {
                ...expressAttrVal,
            },
            xExpressionAttributeNames: {
                ...expressAttrName,
            },
            xFilterExpression: _filterExpressionValue,
        };
        return result;
    }
    operation__filterContains({ fieldName, term }) {
        const attrKeyHash = `#attrKey5${getRandom()}`.toLowerCase();
        const keyAttr = `:attr${fieldName}${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeValues: {
                [keyAttr]: term,
            },
            xExpressionAttributeNames: {
                [attrKeyHash]: fieldName,
            },
            xFilterExpression: `contains (${attrKeyHash}, ${keyAttr})`,
        };
        return result;
    }
    operation__filterBetween({ fieldName, from, to, }) {
        const _attrKeyHash = `#attrKey6${getRandom()}`.toLowerCase();
        const _fromKey = `:fromKey${getRandom()}`.toLowerCase();
        const _toKey = `:toKey${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeValues: {
                [_fromKey]: from,
                [_toKey]: to,
            },
            xExpressionAttributeNames: {
                [_attrKeyHash]: fieldName,
            },
            xFilterExpression: [_attrKeyHash, "between", _fromKey, "and", _toKey].join(" "),
        };
        return result;
    }
    operation__filterBeginsWith({ fieldName, term }) {
        const _attrKeyHash = `#attrKey7${getRandom()}`.toLowerCase();
        const keyAttr = `:attr${fieldName}${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeValues: {
                [keyAttr]: term,
            },
            xExpressionAttributeNames: {
                [_attrKeyHash]: fieldName,
            },
            xFilterExpression: `begins_with (${_attrKeyHash}, ${keyAttr})`,
        };
        return result;
    }
    operation__translateAdvancedQueryOperation({ fieldName, queryObject, }) {
        const queryConditions = [];
        Object.keys(queryObject).forEach((condKey) => {
            const conditionKey = condKey;
            const _conditionObjValue = queryObject[conditionKey];
            if (conditionKey === "$between") {
                if (Array.isArray(_conditionObjValue)) {
                    const _queryConditions = this.operation__filterBetween({
                        fieldName: fieldName,
                        from: _conditionObjValue[0],
                        to: _conditionObjValue[1],
                    });
                    queryConditions.push(_queryConditions);
                }
            }
            else if (conditionKey === "$beginsWith") {
                const _queryConditions = this.operation__filterBeginsWith({
                    fieldName: fieldName,
                    term: _conditionObjValue,
                });
                queryConditions.push(_queryConditions);
            }
            else if (conditionKey === "$contains") {
                const _queryConditions = this.operation__filterContains({
                    fieldName: fieldName,
                    term: _conditionObjValue,
                });
                queryConditions.push(_queryConditions);
            }
            else if (conditionKey === "$in") {
                if (Array.isArray(_conditionObjValue)) {
                    const _queryConditions = this.operation__filterIn({
                        fieldName: fieldName,
                        attrValues: _conditionObjValue,
                    });
                    queryConditions.push(_queryConditions);
                }
            }
            else if (conditionKey === "$notContains") {
                const _queryConditions = this.operation__filterContains({
                    fieldName: fieldName,
                    term: _conditionObjValue,
                });
                _queryConditions.xFilterExpression = `NOT ${_queryConditions.xFilterExpression}`;
                queryConditions.push(_queryConditions);
            }
            else if (conditionKey === "$exists") {
                const _queryConditions = this.operation__filterFieldExist({
                    fieldName: fieldName,
                });
                queryConditions.push(_queryConditions);
            }
            else if (conditionKey === "$notExists") {
                const _queryConditions = this.operation__filterFieldNotExist({
                    fieldName: fieldName,
                });
                queryConditions.push(_queryConditions);
            }
            else {
                if (hasQueryConditionKey(conditionKey)) {
                    const conditionExpr = conditionKeyMap[conditionKey];
                    if (conditionExpr) {
                        const _queryConditions = this.ddo__helperFilterBasic({
                            fieldName: fieldName,
                            val: _conditionObjValue,
                            conditionExpr: conditionExpr,
                        });
                        queryConditions.push(_queryConditions);
                    }
                }
            }
        });
        return queryConditions;
    }
    operation_translateBasicQueryOperation({ fieldName, queryObject }) {
        const _queryConditions = this.ddo__helperFilterBasic({
            fieldName: fieldName,
            val: queryObject,
            conditionExpr: "=",
        });
        return _queryConditions;
    }
    ddo__helperDynamoFilterOperation({ queryDefs, projectionFields, }) {
        let queryAndConditions = [];
        let queryOrConditions = [];
        let _projectionExpression = undefined;
        const andFilterExpressionArray = [];
        const orFilterExpressionArray = [];
        let _expressionAttributeValues = {};
        let _expressionAttributeNames = {};
        Object.keys(queryDefs).forEach((fieldName_or_$or) => {
            if (fieldName_or_$or === "$or") {
                const $orKey = fieldName_or_$or;
                const orArray = queryDefs[$orKey];
                if (Array.isArray(orArray)) {
                    orArray.forEach((orQuery) => {
                        Object.keys(orQuery).forEach((fieldName) => {
                            const orQueryObjectOrValue = orQuery[fieldName];
                            if (typeof orQueryObjectOrValue === "object") {
                                const _orQueryCond = this.operation__translateAdvancedQueryOperation({
                                    fieldName,
                                    queryObject: orQueryObjectOrValue,
                                });
                                queryOrConditions = [...queryOrConditions, ..._orQueryCond];
                            }
                            else {
                                const _orQueryConditions = this.operation_translateBasicQueryOperation({
                                    fieldName,
                                    queryObject: orQueryObjectOrValue,
                                });
                                queryOrConditions = [...queryOrConditions, _orQueryConditions];
                            }
                        });
                    });
                }
            }
            else {
                const fieldName2 = fieldName_or_$or;
                const queryObjectOrValue = queryDefs[fieldName2];
                if (typeof queryObjectOrValue === "object") {
                    const _queryCond = this.operation__translateAdvancedQueryOperation({
                        fieldName: fieldName2,
                        queryObject: queryObjectOrValue,
                    });
                    queryAndConditions = [...queryAndConditions, ..._queryCond];
                }
                else {
                    const _queryConditions = this.operation_translateBasicQueryOperation({
                        fieldName: fieldName2,
                        queryObject: queryObjectOrValue,
                    });
                    queryAndConditions = [...queryAndConditions, _queryConditions];
                }
            }
        });
        for (const item of queryAndConditions) {
            _expressionAttributeNames = {
                ..._expressionAttributeNames,
                ...item.xExpressionAttributeNames,
            };
            _expressionAttributeValues = {
                ..._expressionAttributeValues,
                ...item.xExpressionAttributeValues,
            };
            andFilterExpressionArray.push(item.xFilterExpression);
        }
        for (const item2 of queryOrConditions) {
            _expressionAttributeNames = {
                ..._expressionAttributeNames,
                ...item2.xExpressionAttributeNames,
            };
            _expressionAttributeValues = {
                ..._expressionAttributeValues,
                ...item2.xExpressionAttributeValues,
            };
            orFilterExpressionArray.push(item2.xFilterExpression);
        }
        let _andfilterExpression = null;
        let _orfilterExpression = null;
        if (andFilterExpressionArray?.length) {
            _andfilterExpression = andFilterExpressionArray.join(" AND ").trim();
        }
        if (orFilterExpressionArray?.length) {
            _orfilterExpression = orFilterExpressionArray.join(" OR ").trim();
        }
        let _filterExpression = "";
        if (_andfilterExpression && _orfilterExpression) {
            _filterExpression = `(${_andfilterExpression}) AND (${_orfilterExpression})`;
        }
        else if (_andfilterExpression) {
            _filterExpression = _andfilterExpression;
        }
        else if (_orfilterExpression) {
            _filterExpression = _orfilterExpression;
        }
        if (projectionFields?.length && Array.isArray(projectionFields)) {
            const _projection_expressionAttributeNames = {};
            projectionFields.forEach((field) => {
                if (typeof field === "string") {
                    const attrKeyHash = `#attrKey8${getRandom()}`.toLowerCase();
                    _projection_expressionAttributeNames[attrKeyHash] = field;
                }
            });
            _projectionExpression = Object.keys(_projection_expressionAttributeNames).join(", ");
            _expressionAttributeNames = {
                ..._projection_expressionAttributeNames,
                ..._expressionAttributeNames,
            };
        }
        const _expressionAttributeValuesFinal = UtilService.objectHasAnyProperty(_expressionAttributeValues)
            ? _expressionAttributeValues
            : undefined;
        const _expressionAttributeNamesFinal = UtilService.objectHasAnyProperty(_expressionAttributeNames)
            ? _expressionAttributeNames
            : undefined;
        const queryExpressions = {
            expressionAttributeValues: _expressionAttributeValuesFinal,
            filterExpression: _filterExpression,
            projectionExpressionAttr: _projectionExpression,
            expressionAttributeNames: _expressionAttributeNamesFinal,
        };
        LoggingService.log({ queryExpressions });
        return queryExpressions;
    }
}

==> dynamodb-data-operation-1.0.47.tgz: package/dist-src/core/dynamo-filter-query-operation.js <==
import { LoggingService } from "../helpers/logging-service";
import { UtilService } from "../helpers/util-service";
const conditionKeyMap = {
    $eq: "=",
    $notEq: "<>",
    $lt: "<",
    $lte: "<=",
    $gt: ">",
    $gte: ">=",
    $exists: "",
    $notExists: "",
    $in: "",
    $between: "",
    $contains: "",
    $notContains: "",
    $beginsWith: "",
};
function hasQueryConditionKey(key) {
    const queryCondKeys = Object.keys(conditionKeyMap);
    return queryCondKeys.includes(key);
}
const getRandom = () => [Math.round(Math.random() * 99999), Math.round(Math.random() * 88888), Math.round(Math.random() * 99)].join("");
export class DynamoFilterQueryOperation {
    operation__filterFieldExist({ fieldName }) {
        const attrKeyHash = `#attrKey1${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeNames: {
                [attrKeyHash]: fieldName,
            },
            xFilterExpression: `attribute_exists (${attrKeyHash})`,
        };
        return result;
    }
    operation__filterFieldNotExist({ fieldName }) {
        const attrKeyHash = `#attrKey2${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeNames: {
                [attrKeyHash]: fieldName,
            },
            xFilterExpression: `attribute_not_exists (${attrKeyHash})`,
        };
        return result;
    }
    ddo__helperFilterBasic({ fieldName, val, conditionExpr, }) {
        const keyAttr = `:attr${fieldName}${getRandom()}`.toLowerCase();
        const attrKeyHash = `#attrKey3${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeValues: {
                [keyAttr]: val,
            },
            xExpressionAttributeNames: {
                [attrKeyHash]: fieldName,
            },
            xFilterExpression: [attrKeyHash, conditionExpr, keyAttr].join(" "),
        };
        return result;
    }
    operation__filterIn({ fieldName, attrValues }) {
        const expressAttrVal = {};
        const expressAttrName = {};
        const filterExpress = [];
        const _attrKeyHash = `#attrKey4${getRandom()}`.toLowerCase();
        expressAttrName[_attrKeyHash] = fieldName;
        attrValues.forEach((item) => {
            const keyAttr = `:attr${fieldName}${getRandom()}`.toLowerCase();
            expressAttrVal[keyAttr] = item;
            filterExpress.push(`${_attrKeyHash} = ${keyAttr}`);
        });
        const _filterExpression = filterExpress.join(" OR ").trim();
        const _filterExpressionValue = filterExpress.length > 1 ? `(${_filterExpression})` : _filterExpression;
        const result = {
            xExpressionAttributeValues: {
                ...expressAttrVal,
            },
            xExpressionAttributeNames: {
                ...expressAttrName,
            },
            xFilterExpression: _filterExpressionValue,
        };
        return result;
    }
    operation__filterContains({ fieldName, term }) {
        const attrKeyHash = `#attrKey5${getRandom()}`.toLowerCase();
        const keyAttr = `:attr${fieldName}${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeValues: {
                [keyAttr]: term,
            },
            xExpressionAttributeNames: {
                [attrKeyHash]: fieldName,
            },
            xFilterExpression: `contains (${attrKeyHash}, ${keyAttr})`,
        };
        return result;
    }
    operation__filterBetween({ fieldName, from, to, }) {
        const _attrKeyHash = `#attrKey6${getRandom()}`.toLowerCase();
        const _fromKey = `:fromKey${getRandom()}`.toLowerCase();
        const _toKey = `:toKey${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeValues: {
                [_fromKey]: from,
                [_toKey]: to,
            },
            xExpressionAttributeNames: {
                [_attrKeyHash]: fieldName,
            },
            xFilterExpression: [_attrKeyHash, "between", _fromKey, "and", _toKey].join(" "),
        };
        return result;
    }
    operation__filterBeginsWith({ fieldName, term }) {
        const _attrKeyHash = `#attrKey7${getRandom()}`.toLowerCase();
        const keyAttr = `:attr${fieldName}${getRandom()}`.toLowerCase();
        const result = {
            xExpressionAttributeValues: {
                [keyAttr]: term,
            },
            xExpressionAttributeNames: {
                [_attrKeyHash]: fieldName,
            },
            xFilterExpression: `begins_with (${_attrKeyHash}, ${keyAttr})`,
        };
        return result;
    }
    operation__translateAdvancedQueryOperation({ fieldName, queryObject, }) {
        const queryConditions = [];
        Object.keys(queryObject).forEach((condKey) => {
            const conditionKey = condKey;
            const _conditionObjValue = queryObject[conditionKey];
            if (conditionKey === "$between") {
                if (Array.isArray(_conditionObjValue)) {
                    const _queryConditions = this.operation__filterBetween({
                        fieldName: fieldName,
                        from: _conditionObjValue[0],
                        to: _conditionObjValue[1],
                    });
                    queryConditions.push(_queryConditions);
                }
            }
            else if (conditionKey === "$beginsWith") {
                const _queryConditions = this.operation__filterBeginsWith({
                    fieldName: fieldName,
                    term: _conditionObjValue,
                });
                queryConditions.push(_queryConditions);
            }
            else if (conditionKey === "$contains") {
                const _queryConditions = this.operation__filterContains({
                    fieldName: fieldName,
                    term: _conditionObjValue,
                });
                queryConditions.push(_queryConditions);
            }
            else if (conditionKey === "$in") {
                if (Array.isArray(_conditionObjValue)) {
                    const _queryConditions = this.operation__filterIn({
                        fieldName: fieldName,
                        attrValues: _conditionObjValue,
                    });
                    queryConditions.push(_queryConditions);
                }
            }
            else if (conditionKey === "$notContains") {
                const _queryConditions = this.operation__filterContains({
                    fieldName: fieldName,
                    term: _conditionObjValue,
                });
                _queryConditions.xFilterExpression = `NOT ${_queryConditions.xFilterExpression}`;
                queryConditions.push(_queryConditions);
            }
            else if (conditionKey === "$exists") {
                const _queryConditions = this.operation__filterFieldExist({
                    fieldName: fieldName,
                });
                queryConditions.push(_queryConditions);
            }
            else if (conditionKey === "$notExists") {
                const _queryConditions = this.operation__filterFieldNotExist({
                    fieldName: fieldName,
                });
                queryConditions.push(_queryConditions);
            }
            else {
                if (hasQueryConditionKey(conditionKey)) {
                    const conditionExpr = conditionKeyMap[conditionKey];
                    if (conditionExpr) {
                        const _queryConditions = this.ddo__helperFilterBasic({
                            fieldName: fieldName,
                            val: _conditionObjValue,
                            conditionExpr: conditionExpr,
                        });
                        queryConditions.push(_queryConditions);
                    }
                }
            }
        });
        return queryConditions;
    }
    operation_translateBasicQueryOperation({ fieldName, queryObject }) {
        const _queryConditions = this.ddo__helperFilterBasic({
            fieldName: fieldName,
            val: queryObject,
            conditionExpr: "=",
        });
        return _queryConditions;
    }
    ddo__helperDynamoFilterOperation({ queryDefs, projectionFields, }) {
        let queryAndConditions = [];
        let queryOrConditions = [];
        let _projectionExpression = undefined;
        const andFilterExpressionArray = [];
        const orFilterExpressionArray = [];
        let _expressionAttributeValues = {};
        let _expressionAttributeNames = {};
        Object.keys(queryDefs).forEach((fieldName_Or_And) => {
            if (fieldName_Or_And === "$or") {
                const orKey = fieldName_Or_And;
                const orArray = queryDefs[orKey];
                if (Array.isArray(orArray)) {
                    orArray.forEach((orQuery) => {
                        Object.keys(orQuery).forEach((fieldName) => {
                            const orQueryObjectOrValue = orQuery[fieldName];
                            if (typeof orQueryObjectOrValue === "object") {
                                const _orQueryCond = this.operation__translateAdvancedQueryOperation({
                                    fieldName,
                                    queryObject: orQueryObjectOrValue,
                                });
                                queryOrConditions = [...queryOrConditions, ..._orQueryCond];
                            }
                            else {
                                const _orQueryConditions = this.operation_translateBasicQueryOperation({
                                    fieldName,
                                    queryObject: orQueryObjectOrValue,
                                });
                                queryOrConditions = [...queryOrConditions, _orQueryConditions];
                            }
                        });
                    });
                }
            }
            else if (fieldName_Or_And === "$and") {
                const andKey = fieldName_Or_And;
                const andArray = queryDefs[andKey];
                if (Array.isArray(andArray)) {
                    andArray.forEach((andQuery) => {
                        Object.keys(andQuery).forEach((fieldName) => {
                            const andQueryObjectOrValue = andQuery[fieldName];
                            if (typeof andQueryObjectOrValue === "object") {
                                const _andQueryCond = this.operation__translateAdvancedQueryOperation({
                                    fieldName,
                                    queryObject: andQueryObjectOrValue,
                                });
                                queryAndConditions = [...queryAndConditions, ..._andQueryCond];
                            }
                            else {
                                const _andQueryConditions = this.operation_translateBasicQueryOperation({
                                    fieldName,
                                    queryObject: andQueryObjectOrValue,
                                });
                                queryAndConditions = [...queryAndConditions, _andQueryConditions];
                            }
                        });
                    });
                }
            }
            else {
                const fieldName2 = fieldName_Or_And;
                const queryObjectOrValue = queryDefs[fieldName2];
                if (typeof queryObjectOrValue === "object") {
                    const _queryCond = this.operation__translateAdvancedQueryOperation({
                        fieldName: fieldName2,
                        queryObject: queryObjectOrValue,
                    });
                    queryAndConditions = [...queryAndConditions, ..._queryCond];
                }
                else {
                    const _queryConditions = this.operation_translateBasicQueryOperation({
                        fieldName: fieldName2,
                        queryObject: queryObjectOrValue,
                    });
                    queryAndConditions = [...queryAndConditions, _queryConditions];
                }
            }
        });
        for (const item of queryAndConditions) {
            _expressionAttributeNames = {
                ..._expressionAttributeNames,
                ...item.xExpressionAttributeNames,
            };
            _expressionAttributeValues = {
                ..._expressionAttributeValues,
                ...item.xExpressionAttributeValues,
            };
            andFilterExpressionArray.push(item.xFilterExpression);
        }
        for (const item2 of queryOrConditions) {
            _expressionAttributeNames = {
                ..._expressionAttributeNames,
                ...item2.xExpressionAttributeNames,
            };
            _expressionAttributeValues = {
                ..._expressionAttributeValues,
                ...item2.xExpressionAttributeValues,
            };
            orFilterExpressionArray.push(item2.xFilterExpression);
        }
        let _andfilterExpression = null;
        let _orfilterExpression = null;
        if (andFilterExpressionArray?.length) {
            _andfilterExpression = andFilterExpressionArray.join(" AND ").trim();
        }
        if (orFilterExpressionArray?.length) {
            _orfilterExpression = orFilterExpressionArray.join(" OR ").trim();
        }
        let _filterExpression = "";
        if (_andfilterExpression && _orfilterExpression) {
            _filterExpression = `(${_andfilterExpression}) AND (${_orfilterExpression})`;
        }
        else if (_andfilterExpression) {
            _filterExpression = _andfilterExpression;
        }
        else if (_orfilterExpression) {
            _filterExpression = _orfilterExpression;
        }
        if (projectionFields?.length && Array.isArray(projectionFields)) {
            const _projection_expressionAttributeNames = {};
            projectionFields.forEach((field) => {
                if (typeof field === "string") {
                    const attrKeyHash = `#attrKey8${getRandom()}`.toLowerCase();
                    _projection_expressionAttributeNames[attrKeyHash] = field;
                }
            });
            _projectionExpression = Object.keys(_projection_expressionAttributeNames).join(", ");
            _expressionAttributeNames = {
                ..._projection_expressionAttributeNames,
                ..._expressionAttributeNames,
            };
        }
        const _expressionAttributeValuesFinal = UtilService.objectHasAnyProperty(_expressionAttributeValues)
            ? _expressionAttributeValues
            : undefined;
        const _expressionAttributeNamesFinal = UtilService.objectHasAnyProperty(_expressionAttributeNames)
            ? _expressionAttributeNames
            : undefined;
        const queryExpressions = {
            expressionAttributeValues: _expressionAttributeValuesFinal,
            filterExpression: _filterExpression,
            projectionExpressionAttr: _projectionExpression,
            expressionAttributeNames: _expressionAttributeNamesFinal,
        };
        LoggingService.log({ queryExpressions });
        return queryExpressions;
    }
}
